//
//    StickyKeys for EPOC
//
//    Copyright (C) 2000  Marc Boogaards
//
//    This program is free software; you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation; either version 2 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program; if not, write to the Free Software
//    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//
#if !defined(__STICKYKEYS_H)
#define __STICKYKEYS_H

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#define USE_MODIFIER_STATE
//#define USE_RAW_EVENTS

#define APP_NAME "StickyKeys"

#include <e32std.h>
#include <e32uid.h>
#include <eikappui.h>
#include <eikapp.h>
#include <eikenv.h>
#include <w32std.h>

class CWsClient;
class CWsRedrawer;
class CWindow;

typedef TFullName TMsg;

///////////////////////////////////////////////////////////////////////////////
//	CKeyState
//
//	Implements a simple state machine to keep track of 'sticky state' for a
//	certain modifier key, in response to key presses. The state diagram is
//	included in KeyState.bmp. Implementation of the state changes is rather
//	blunt but effective (see ProcessKey).
///////////////////////////////////////////////////////////////////////////////

class CKeyState
{
public:
	enum {
		KeyDown,
		KeyUp,
		OtherDown,
		OtherUp
	};
	CKeyState();
	~CKeyState();
	void ProcessKey(int Event);
	int Down() const;
	int Sticky() const;
	int Changed();
	void Reset();
private:
	int iState;
	int iChanged;
	enum {
		S0 = 0, // OtherDown=0, Sticky=0, KeyDown=0 = 000
		S1 = 1, // OtherDown=0, Sticky=0, KeyDown=1 = 001
		S2 = 2, // OtherDown=0, Sticky=1, KeyDown=0 = 010
		S3 = 3, // OtherDown=0, Sticky=1, KeyDown=1 = 011
		S4 = 6, // OtherDown=1, Sticky=1, KeyDown=0 = 110
		S5 = 5  // OtherDown=1, Sticky=0, KeyDown=1 = 101
	};
	enum {
		MASK_KEYDOWN = 0x1,
		MASK_STICKY	= 0x2,
		MASK_OTHERKEY = 0x4
	};

};

///////////////////////////////////////////////////////////////////////////////
//	CWindow
//
//	Implements the small message window in the lower left corner of the
//	screen showing the current key state. This window works solely on the
//	Windows Server, and is independent of EIKON.
///////////////////////////////////////////////////////////////////////////////

class CWindow : public CBase
{
public:
	CWindow(CWsClient* aClient);
	void ConstructL (const TMsg &aText, TInt FontSize);
	~CWindow();
	// access
	RWindow& Window(); // our own window
	CWindowGc* SystemGc(); // system graphics context
	// drawing
	void Draw(const TRect& aRect);
	void HandlePointerEvent (TPointerEvent& aPointerEvent);
	void SetText(const TMsg &aText);
protected:
	RWindow iWindow; // window server window
private:
	CWsClient* iClient; // client including session and group
	CFont* iFont;
	TInt iBaselineOffset;
	TInt FontSize;
	TMsg aString;
};

///////////////////////////////////////////////////////////////////////////////
//	CWsRedrawer
//
//	Helper for CWindow
///////////////////////////////////////////////////////////////////////////////

class CWsRedrawer : public CActive
{
public:
	CWsRedrawer();
	~CWsRedrawer();
	void ConstructL(CWsClient* aClient);
	// drawing
	void IssueRequest();
	void DoCancel();
	void RunL();
protected:
	CWsClient* iClient;
};

///////////////////////////////////////////////////////////////////////////////
//	CWsClient
//
//	Implements the Windows Server client, which creates a scheduler and
//	installs a basically the hook that monitors all key events
///////////////////////////////////////////////////////////////////////////////

class CWsClient : public CActive
{
public:
	CWsClient();
	void ConstructL();
	~CWsClient();
	// terminate cleanly
	void Exit();
	// active object protocol
	void IssueRequest(); // request an event
	void DoCancel(); // cancel the request
	void RunL(); // handle completed request
	CWsScreenDevice* iScreen;
	CWsRedrawer* iRedrawer;
private:
	// three state machines, one for each modifier:
	CKeyState CtrlState;
	CKeyState ShiftState;
	CKeyState FuncState;

	TWsEvent iWsEvent;
	CWindow *iMsgBox;
	RWindowGroup iGroup;
	RWsSession iWs;
	CWindowGc* iGc;			// the graphics context
	RMutex iGlobalMutex;	// a global mutex, only used to determine if another
							// copy of us is already in memory.
	TInt32 hCaptureCtrl[256];// capture handles for all keys
	friend class CWindow;
	friend class CWsRedrawer;
};

#endif // !defined(__STICKYKEYS_H)
